/*
 *  skeleton 3rd party pygame module
 */
#include "pygame/pygame.h"
#include "SDL_image.h"


/** these init functions are only needed if your module requires
 ** specific startup/shutdown functions. by using the autoinit
 ** and autoquit functions like this, you can have pygame auto-
 ** matically init()/quit() your module when pygame.init() is called.
 ** if you do not need these initializations, you can delete all the
 ** way down to the "end of inits" comment.
 **/
static int module_initialized = 0;

static void module_autoquit(void)
{
	if(module_initialized)
	{
		module_initialized = 0;
	}
}


static PyObject* module_autoinit(PyObject* self, PyObject* arg) 
{
	if(!PyArg_ParseTuple(arg, ""))
		return NULL;
	if(!module_initialized)
	{
		module_initialized = 1;
		PyGame_RegisterQuit(module_autoquit);
	}
	return PyInt_FromLong(1);
}


	static char doc_quit[] =
		"pygame.module.quit() -> none\n"
		"uninitialize the module\n"
		"\n"
		"Manually uninitialize the module. It is safe to call\n"
		"this if it is currently not initialized.\n";
static PyObject* quit(PyObject* self, PyObject* arg) 
{
	if(!PyArg_ParseTuple(arg, ""))
		return NULL;
	module_autoquit();
	RETURN_NONE
}


	static char doc_init[] =
		"pygame.module.init() -> None\n"
		"initialize the module\n"
		"\n"
		"Manually initialize the module. Will raise an exception if\n"
		"it cannot be initialized. It is safe to call this function if\n"
		"it is currently initialized.\n";
static PyObject* init( PyObject* self, PyObject* arg )
{
	PyObject* result;
	int istrue;
	if(!PyArg_ParseTuple(arg, ""))
		return NULL;
	
	result = module_autoinit(self, arg);
	istrue = PyObject_IsTrue(result);
	Py_DECREF( result );
	if(!istrue)
		return RAISE(PyExc_SDLError, SDL_GetError());
	
	RETURN_NONE
}

	static char doc_get_init[] =
		"pygame.module.get_init() -> bool\n"
		"get status of module initialization\n"
		"\n"
		"Returns true if the module is currently intialized.\n";
static PyObject* get_init(PyObject* self, PyObject* arg)
{
	if(!PyArg_ParseTuple( arg, "" )) return NULL;
	return PyInt_FromLong(module_initialized);
}

/** End of Init Functions **/





static char doc_samplefunc[] =
   "samplefunc(Surface) -> Rect\n"
   "crude sample function\n"
   "\n"
   "this will simply erase a surface with solid black. returns\n"
   "a rect covering the whole surface.\n";

static PyObject* samplefunc(PyObject* self, PyObject* arg)
{
	int result;
	PyObject* surfobj;
	SDL_Surface* surf;

	if(!PyArg_ParseTuple(arg, "O!", &PySurface_Type, &surfobj))
		return NULL;
	surf = PySurface_AsSurface(surfobj);

	PySurface_Prep(surfobj);
	result = SDL_FillRect(surf, NULL, 0);
	PySurface_Unprep(surfobj);

	if(result == -1)
		return RAISE(PyExc_SDLError, SDL_GetError());

	return PyRect_New4(0, 0, (short)surf->w, (short)surf->h);
}



/*list of functions in the module*/

static PyMethodDef module_functions[] = {
/*again, no need for these init functions if you don't use them
*the __PYGAMEinit__ is the magic function that gets auto init'd*/
	{ "__PYGAMEinit__", module_autoinit, 1, doc_init },
	{ "init", init, 1, doc_init },
	{ "quit", quit, 1, doc_quit },
	{ "get_init", get_init, 1, doc_get_init },
	
	{ "samplefunc", samplefunc, 1, doc_samplefunc },
	
	{ NULL, NULL }
};


static char doc_MODULE[] =
   "Documentation for the exciting skeleton sample\n";


/* python init function goes here, must follow all rules
 * for a python init-stub function
 */
void initskeleton()
{
	PyObject *module;
	
    /* create the module */
	module = Py_InitModule3( "skeleton", module_functions, doc_MODULE );
	
	/*imported needed pygame apis*/
	import_pygame_base();
	import_pygame_surface();
	import_pygame_rect();
}

